@@ -32,6 +32,8 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreAssimpLoader.h"
+#include "OgreString.h"
+#include "OgreStringConverter.h"
 
 #include <assimp/version.h>
 #include <assimp/scene.h>
@@ -329,8 +331,6 @@ String ReplaceSpaces(const String& s)
 }
 } // namespace
 
-int AssimpLoader::msBoneCount = 0;
-
 AssimpLoader::AssimpLoader()
 {
     Assimp::DefaultLogger::create("");
@@ -360,15 +360,41 @@ bool AssimpLoader::load(const String& source, Mesh* mesh, SkeletonPtr& skeletonP
 bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mesh, SkeletonPtr& skeletonPtr,
                          const Options& options)
 {
+    mAnimationSpeedModifier = options.animationSpeedModifier;
+    mLoaderParams = options.params;
+    mCustomAnimationName = options.customAnimationName;
+
+    float maxEdgeAngle = options.maxEdgeAngle;
+    int postProcessSteps = options.postProcessSteps;
+    auto optsAny = mesh->getUserObjectBindings().getUserAny("_AssimpLoaderOptions");
+    if(optsAny.has_value())
+    {
+        auto strOpts = any_cast<BinaryOptionList>(optsAny);
+        if(strOpts.find("quiet") != strOpts.end())
+            mLoaderParams |= LP_QUIET_MODE;
+        if(strOpts.find("customAnimationName") != strOpts.end())
+            mCustomAnimationName = strOpts["customAnimationName"];
+        if(strOpts.find("animationSpeedModifier") != strOpts.end())
+            StringConverter::parse(strOpts["animationSpeedModifier"], mAnimationSpeedModifier);
+        if(strOpts.find("maxEdgeAngle") != strOpts.end())
+            StringConverter::parse(strOpts["maxEdgeAngle"], maxEdgeAngle);
+        if(strOpts.find("postProcessSteps") != strOpts.end())
+            StringConverter::parse(strOpts["postProcessSteps"], postProcessSteps);
+        if(strOpts.find("cutAnimation") != strOpts.end())
+            mLoaderParams |= LP_CUT_ANIMATION_WHERE_NO_FURTHER_CHANGE;
+    }
+
     uint32 flags = aiProcessPreset_TargetRealtime_Fast | aiProcess_TransformUVCoords | aiProcess_FlipUVs;
     flags &= ~(aiProcess_JoinIdenticalVertices | aiProcess_CalcTangentSpace); // optimize for fast loading
 
-    flags |= options.postProcessSteps;
+    flags |= postProcessSteps;
 
     if((flags & (aiProcess_GenSmoothNormals | aiProcess_GenNormals)) != aiProcess_GenNormals)
         flags &= ~aiProcess_GenNormals; // prefer smooth normals
 
-    importer.SetPropertyFloat("PP_GSN_MAX_SMOOTHING_ANGLE", options.maxEdgeAngle);
+    importer.SetPropertyFloat("PP_GSN_MAX_SMOOTHING_ANGLE", maxEdgeAngle);
+    importer.SetPropertyBool(AI_CONFIG_IMPORT_FBX_EMBEDDED_TEXTURES_LEGACY_NAMING, true);
+
     const aiScene* scene = importer.ReadFile(name, flags);
 
     // If the import failed, report it
@@ -378,10 +404,7 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
         return false;
     }
 
-    mAnimationSpeedModifier = options.animationSpeedModifier;
-    mLoaderParams = options.params;
     mQuietMode = mLoaderParams & LP_QUIET_MODE;
-    mCustomAnimationName = options.customAnimationName;
     mNodeDerivedTransformByName.clear();
 
     String basename, extension;
@@ -394,11 +417,9 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
 
     if (mBonesByName.size())
     {
-        mSkeleton = SkeletonManager::getSingleton().create(basename + ".skeleton", RGN_DEFAULT, true);
+        mSkeleton = SkeletonManager::getSingleton().create(basename + ".skeleton", mesh->getGroup(), true);
 
-        msBoneCount = 0;
         createBonesFromNode(scene, scene->mRootNode);
-        msBoneCount = 0;
         createBoneHiearchy(scene, scene->mRootNode);
 
         if (scene->HasAnimations())
@@ -415,7 +436,7 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
     {
         const aiTexture* tex = scene->mTextures[i];
         auto texname =
-            StringUtil::format("%s%s.%s", mesh->getName().c_str(), tex->mFilename.C_Str(), tex->achFormatHint);
+            StringUtil::format("%s%s%d.%s", mesh->getName().c_str(), tex->mFilename.C_Str(), i, tex->achFormatHint);
         if (TextureManager::getSingleton().resourceExists(texname, mesh->getGroup()))
             continue;
 
@@ -441,7 +462,10 @@ bool AssimpLoader::_load(const char* name, Assimp::Importer& importer, Mesh* mes
         TextureManager::getSingleton().loadImage(texname, mesh->getGroup(), img);
     }
 
-    loadDataFromNode(scene, scene->mRootNode, mesh);
+    auto aabb = loadDataFromNode(scene, scene->mRootNode, mesh);
+    // We must indicate the bounding box
+    mesh->_setBounds(aabb);
+    mesh->_setBoundingSphereRadius((aabb.getMaximum() - aabb.getMinimum()).length() / 2);
 
     Assimp::DefaultLogger::kill();
 
@@ -736,7 +760,7 @@ void AssimpLoader::createBonesFromNode(const aiScene* mScene, const aiNode* pNod
 {
     if (isNodeNeeded(pNode->mName.data))
     {
-        Bone* bone = mSkeleton->createBone(String(pNode->mName.data), msBoneCount);
+        Bone* bone = mSkeleton->createBone(String(pNode->mName.data));
 
         aiQuaternion rot;
         aiVector3D pos;
@@ -753,10 +777,9 @@ void AssimpLoader::createBonesFromNode(const aiScene* mScene, const aiNode* pNod
 
         if (!mQuietMode)
         {
-            LogManager::getSingleton().logMessage(StringConverter::toString(msBoneCount) +
+            LogManager::getSingleton().logMessage(StringConverter::toString(bone->getHandle()) +
                                                   ") Creating bone '" + String(pNode->mName.data) + "'");
         }
-        msBoneCount++;
     }
     // Traverse all child nodes of the current node instance
     for (unsigned int childIdx = 0; childIdx < pNode->mNumChildren; ++childIdx)
@@ -888,7 +911,8 @@ static bool getTextureName(const aiMaterial* mat, aiTextureType type, const aiSc
         const aiTexture* tex = scene->GetEmbeddedTexture(path.C_Str());
         if(tex)
         {
-            basename = StringUtil::format("%s%s.%.8s", meshName.c_str(), tex->mFilename.C_Str(), tex->achFormatHint);
+            basename = StringUtil::format("%s%s%s.%.8s", meshName.c_str(), tex->mFilename.C_Str(), path.C_Str() + 1,
+                                          tex->achFormatHint);
             return true;
         }
 
@@ -939,6 +963,13 @@ static MaterialPtr createMaterial(const aiMaterial* mat, const Ogre::String &gro
         LogManager::getSingleton().logMessage("Creating " + matName);
     }
 
+    aiString tmp;
+    String alphaMode;
+    if (AI_SUCCESS == aiGetMaterialString(mat, "$mat.gltf.alphaMode", 0, 0, &tmp))
+    {
+        alphaMode = String(tmp.data);
+    }
+
     // ambient
     aiColor4D clr(1.0f, 1.0f, 1.0f, 1.0);
     // Ambient is usually way too low! FIX ME!
@@ -953,6 +984,12 @@ static MaterialPtr createMaterial(const aiMaterial* mat, const Ogre::String &gro
         omat->setDiffuse(clr.r, clr.g, clr.b, clr.a);
     }
 
+    if (clr.a < 1.0f || alphaMode == "BLEND")
+    {
+        omat->setSceneBlending(SBT_TRANSPARENT_ALPHA);
+        omat->setDepthWriteEnabled(false);
+    }
+
     // specular
     clr = aiColor4D(1.0f, 1.0f, 1.0f, 1.0f);
     if (AI_SUCCESS == aiGetMaterialColor(mat, AI_MATKEY_COLOR_SPECULAR, &clr))
@@ -993,6 +1030,12 @@ static MaterialPtr createMaterial(const aiMaterial* mat, const Ogre::String &gro
         }
     }
 
+    float alphaCutoff = 1.0f;
+    if (AI_SUCCESS == mat->Get("$mat.gltf.alphaCutoff", 0, 0, alphaCutoff) && alphaMode == "MASK")
+    {
+        omat->getTechnique(0)->getPass(0)->setAlphaRejectSettings(CMPF_GREATER, alphaCutoff * 255.0f);
+    }
+
     String basename;
     if (getTextureName(mat, type, scene, meshName, basename))
     {
@@ -1086,10 +1129,12 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
     // prime pointers to vertex related data
     aiVector3D* vec = mesh->mVertices;
     aiVector3D* norm = mesh->mNormals;
-    aiVector3D* uv = mesh->mTextureCoords[0];
     aiVector3D* tang = mesh->mTangents;
     aiColor4D *col = mesh->mColors[0];
 
+    if(!norm)
+        matptr->setLightingEnabled(false);
+
     // We must create the vertex data, indicating how many vertices there will be
     submesh->createVertexData();
     submesh->vertexData->vertexStart = 0;
@@ -1128,13 +1173,16 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
         offset += declaration->addElement(source, offset, VET_FLOAT3, VES_NORMAL).getSize();
     }
 
-    if (uv)
+    for (int uvindex = 0; uvindex < AI_MAX_NUMBER_OF_TEXTURECOORDS; uvindex++)
     {
+        aiVector3D* uv = mesh->mTextureCoords[uvindex];
+        if (!uv)
+            break;
         if (!mQuietMode)
         {
             LogManager::getSingleton().logMessage(StringUtil::format("%d uvs", mesh->mNumVertices));
         }
-        offset += declaration->addElement(source, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES).getSize();
+        offset += declaration->addElement(source, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES, uvindex).getSize();
     }
 
     if (tang)
@@ -1203,11 +1251,13 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
         }
 
         // uvs
-        if (uv)
+        for (int uvindex = 0; uvindex < AI_MAX_NUMBER_OF_TEXTURECOORDS; uvindex++)
         {
-            *vdata++ = uv->x;
-            *vdata++ = uv->y;
-            uv++;
+            aiVector3D* uv = mesh->mTextureCoords[uvindex];
+            if (!uv)
+                break;
+            *vdata++ = uv[i].x;
+            *vdata++ = uv[i].y;
         }
 
         if(tang)
@@ -1307,12 +1357,11 @@ bool AssimpLoader::createSubMesh(const String& name, int index, const aiNode* pN
     return true;
 }
 
-void AssimpLoader::loadDataFromNode(const aiScene* mScene, const aiNode* pNode, Mesh* mesh)
+AxisAlignedBox AssimpLoader::loadDataFromNode(const aiScene* mScene, const aiNode* pNode, Mesh* mesh)
 {
+    AxisAlignedBox aabb;
     if (pNode->mNumMeshes > 0)
     {
-        AxisAlignedBox mAAB = mesh->getBounds();
-
         for (unsigned int idx = 0; idx < pNode->mNumMeshes; ++idx)
         {
             aiMesh* pAIMesh = mScene->mMeshes[pNode->mMeshes[idx]];
@@ -1325,20 +1374,18 @@ void AssimpLoader::loadDataFromNode(const aiScene* mScene, const aiNode* pNode,
             // Create a material instance for the mesh.
             const aiMaterial* pAIMaterial = mScene->mMaterials[pAIMesh->mMaterialIndex];
             MaterialPtr matptr = createMaterial(pAIMaterial, mesh->getGroup(), mesh->getName(), mScene, !mQuietMode);
-            createSubMesh(pNode->mName.data, idx, pNode, pAIMesh, matptr, mesh, mAAB);
+            createSubMesh(pNode->mName.data, idx, pNode, pAIMesh, matptr, mesh, aabb);
         }
-
-        // We must indicate the bounding box
-        mesh->_setBounds(mAAB);
-        mesh->_setBoundingSphereRadius((mAAB.getMaximum() - mAAB.getMinimum()).length() / 2);
     }
 
     // Traverse all child nodes of the current node instance
     for (unsigned int childIdx = 0; childIdx < pNode->mNumChildren; childIdx++)
     {
         const aiNode* pChildNode = pNode->mChildren[childIdx];
-        loadDataFromNode(mScene, pChildNode, mesh);
+        aabb.merge(loadDataFromNode(mScene, pChildNode, mesh));
     }
+
+    return aabb;
 }
 
 static std::vector<std::unique_ptr<Codec> > registeredCodecs;
@@ -1363,7 +1410,7 @@ struct AssimpCodec : public Codec
 
     static void startup()
     {
-        String version = StringUtil::format("Assimp - %d.%d.%d - Open-Asset-Importer", aiGetVersionMajor(),
+        String version = StringUtil::format("Assimp - %d.%d.%x - Open-Asset-Importer", aiGetVersionMajor(),
                                             aiGetVersionMinor(), aiGetVersionRevision());
         LogManager::getSingleton().logMessage(version);
 
