@@ -130,7 +130,7 @@ struct DotSceneCodec : public Codec
 
 } // namespace
 
-DotSceneLoader::DotSceneLoader() : mSceneMgr(0), mBackgroundColour(ColourValue::Black) {}
+DotSceneLoader::DotSceneLoader() : mSceneMgr(0), mBackgroundColour(ColourValue::Black), mItemPrefix("") {}
 
 DotSceneLoader::~DotSceneLoader() {}
 
@@ -141,6 +141,18 @@ void DotSceneLoader::load(DataStreamPtr& stream, const String& groupName, SceneN
 
     pugi::xml_document XMLDoc; // character type defaults to char
 
+    auto optsAny = rootNode->getUserObjectBindings().getUserAny("_DotSceneLoaderOptions");
+    if (optsAny.has_value())
+    {
+        std::map<Ogre::String, Ogre::Any> options = any_cast<std::map<Ogre::String, Ogre::Any>>(optsAny);
+        if (options.find("namePrefix") != options.end())
+        {
+            Ogre::Any optionValue = options["namePrefix"];
+            if (optionValue.has_value())
+                mItemPrefix = Ogre::any_cast<Ogre::String>(optionValue);
+        }
+    }
+
     auto result = XMLDoc.load_buffer(stream->getAsString().c_str(), stream->size());
     if (!result)
     {
@@ -223,21 +235,18 @@ void DotSceneLoader::processNodes(pugi::xml_node& XMLNode)
     if (auto pElement = XMLNode.child("position"))
     {
         mAttachNode->setPosition(parseVector3(pElement));
-        mAttachNode->setInitialState();
     }
 
     // Process rotation (?)
     if (auto pElement = XMLNode.child("rotation"))
     {
         mAttachNode->setOrientation(parseQuaternion(pElement));
-        mAttachNode->setInitialState();
     }
 
     // Process scale (?)
     if (auto pElement = XMLNode.child("scale"))
     {
         mAttachNode->setScale(parseVector3(pElement));
-        mAttachNode->setInitialState();
     }
 }
 
@@ -338,6 +347,8 @@ void DotSceneLoader::processLight(pugi::xml_node& XMLNode, SceneNode* pParent)
         pLight->setType(Light::LT_SPOTLIGHT);
     else if (sValue == "radPoint")
         pLight->setType(Light::LT_POINT);
+    else if (sValue == "rect")
+        pLight->setType(Light::LT_RECTLIGHT);
 
     pLight->setVisible(getAttribBool(XMLNode, "visible", true));
     pLight->setCastShadows(getAttribBool(XMLNode, "castShadows", true));
@@ -361,6 +372,14 @@ void DotSceneLoader::processLight(pugi::xml_node& XMLNode, SceneNode* pParent)
         if (auto pElement = XMLNode.child("lightAttenuation"))
             processLightAttenuation(pElement, pLight);
     }
+
+    if (sValue == "rect")
+    {
+        // Process lightSourceSize (?)
+        if (auto pElement = XMLNode.child("lightSourceSize"))
+            processLightSourceSize(pElement, pLight);
+	}
+
     // Process userDataReference (?)
     if (auto pElement = XMLNode.child("userData"))
         processUserData(pElement, pLight->getUserObjectBindings());
@@ -435,9 +454,9 @@ void DotSceneLoader::processNode(pugi::xml_node& XMLNode, SceneNode* pParent)
     {
         // Provide the name
         if (pParent)
-            pNode = pParent->createChildSceneNode(name);
+            pNode = pParent->createChildSceneNode(mItemPrefix + name);
         else
-            pNode = mAttachNode->createChildSceneNode(name);
+            pNode = mAttachNode->createChildSceneNode(mItemPrefix + name);
     }
 
     // Process other attributes
@@ -446,21 +465,18 @@ void DotSceneLoader::processNode(pugi::xml_node& XMLNode, SceneNode* pParent)
     if (auto pElement = XMLNode.child("position"))
     {
         pNode->setPosition(parseVector3(pElement));
-        pNode->setInitialState();
     }
 
     // Process rotation (?)
     if (auto pElement = XMLNode.child("rotation"))
     {
         pNode->setOrientation(parseQuaternion(pElement));
-        pNode->setInitialState();
     }
 
     // Process scale (?)
     if (auto pElement = XMLNode.child("scale"))
     {
         pNode->setScale(parseVector3(pElement));
-        pNode->setInitialState();
     }
 
     // Process lookTarget (?)
@@ -601,59 +617,64 @@ void DotSceneLoader::processEntity(pugi::xml_node& XMLNode, SceneNode* pParent)
     // Process attributes
     String name = getAttrib(XMLNode, "name");
 
-    LogManager::getSingleton().logMessage("[DotSceneLoader] Processing Entity: " + name, LML_TRIVIAL);
+    LogManager::getSingleton().logMessage("[DotSceneLoader] Processing Entity: " + mItemPrefix + name, LML_TRIVIAL);
 
     String meshFile = getAttrib(XMLNode, "meshFile");
-	String staticGeometry = getAttrib(XMLNode, "static");
-	String instancedManager = getAttrib(XMLNode, "instanced");
-	String material = getAttrib(XMLNode, "material");
+    String staticGeometry = getAttrib(XMLNode, "static");
+    String instancedManager = getAttrib(XMLNode, "instanced");
+    String material = getAttrib(XMLNode, "material");
     bool castShadows = getAttribBool(XMLNode, "castShadows", true);
     bool visible = getAttribBool(XMLNode, "visible", true);
 
     // Create the entity
-	MovableObject* pEntity = 0;
+    MovableObject* pEntity = 0;
 
     try
     {
-		// If the Entity is instanced then the creation path is different
-		if (!instancedManager.empty())
-		{
-			LogManager::getSingleton().logMessage("[DotSceneLoader] Adding entity: " + name + " to Instance Manager: " + instancedManager, LML_TRIVIAL);
+        // If the Entity is instanced then the creation path is different
+        if (!instancedManager.empty())
+        {
+            LogManager::getSingleton().logMessage(
+                "[DotSceneLoader] Adding entity: " + name + " to Instance Manager: " + instancedManager, LML_TRIVIAL);
 
-			// Load the Mesh to get the material name of the first submesh
-			Ogre::MeshPtr mesh = MeshManager::getSingletonPtr()->load(meshFile, m_sGroupName);
+            // Load the Mesh to get the material name of the first submesh
+            Ogre::MeshPtr mesh = MeshManager::getSingletonPtr()->load(meshFile, m_sGroupName);
 
-			// Get the material name of the entity
-			if(!material.empty())
-				pEntity = mSceneMgr->createInstancedEntity(material, instancedManager);
-			else
-				pEntity = mSceneMgr->createInstancedEntity(mesh->getSubMesh(0)->getMaterialName(), instancedManager);
+            // Get the material name of the entity
+            if (!material.empty())
+                pEntity = mSceneMgr->createInstancedEntity(material, instancedManager);
+            else
+                pEntity = mSceneMgr->createInstancedEntity(mesh->getSubMesh(0)->getMaterialName(), instancedManager);
 
-			pParent->attachObject(static_cast<InstancedEntity*>(pEntity));
-		}
-		else
-		{
-			pEntity = mSceneMgr->createEntity(name, meshFile, m_sGroupName);
+            pParent->attachObject(static_cast<InstancedEntity*>(pEntity));
+        }
+        else
+        {
+            pEntity = mSceneMgr->createEntity(mItemPrefix + name, meshFile, m_sGroupName);
 
-			static_cast<Entity*>(pEntity)->setCastShadows(castShadows);
-			static_cast<Entity*>(pEntity)->setVisible(visible);
+            static_cast<Entity*>(pEntity)->setCastShadows(castShadows);
+            static_cast<Entity*>(pEntity)->setVisible(visible);
 
-			if (!material.empty())
-				static_cast<Entity*>(pEntity)->setMaterialName(material);
+            if (!material.empty())
+                static_cast<Entity*>(pEntity)->setMaterialName(material);
 
-			// If the Entity belongs to a Static Geometry group then it doesn't get attached to a node
-			// * TODO * : Clean up nodes without attached entities or children nodes? (should be done afterwards if the hierarchy is being processed)
-			if (!staticGeometry.empty())
-			{
-				LogManager::getSingleton().logMessage("[DotSceneLoader] Adding entity: " + name + " to Static Group: " + staticGeometry, LML_TRIVIAL);
-				mSceneMgr->getStaticGeometry(staticGeometry)->addEntity(static_cast<Entity*>(pEntity), pParent->_getDerivedPosition(), pParent->_getDerivedOrientation(), pParent->_getDerivedScale());
-			}
-			else
-			{
-				LogManager::getSingleton().logMessage("[DotSceneLoader] pParent->attachObject(): " + name, LML_TRIVIAL);
-				pParent->attachObject(static_cast<Entity*>(pEntity));
-			}
-		}
+            // If the Entity belongs to a Static Geometry group then it doesn't get attached to a node
+            // * TODO * : Clean up nodes without attached entities or children nodes? (should be done afterwards
+            // if the hierarchy is being processed)
+            if (!staticGeometry.empty())
+            {
+                LogManager::getSingleton().logMessage(
+                    "[DotSceneLoader] Adding entity: " + name + " to Static Group: " + staticGeometry, LML_TRIVIAL);
+                mSceneMgr->getStaticGeometry(staticGeometry)
+                    ->addEntity(static_cast<Entity*>(pEntity), pParent->_getDerivedPosition(),
+                                pParent->_getDerivedOrientation(), pParent->_getDerivedScale());
+            }
+            else
+            {
+                LogManager::getSingleton().logMessage("[DotSceneLoader] pParent->attachObject(): " + name, LML_TRIVIAL);
+                pParent->attachObject(static_cast<Entity*>(pEntity));
+            }
+        }
     }
     catch (const Exception& e)
     {
@@ -765,9 +786,6 @@ void DotSceneLoader::processSkyBox(pugi::xml_node& XMLNode)
     String material = getAttrib(XMLNode, "material", "BaseWhite");
     Real distance = getAttribReal(XMLNode, "distance", 5000);
     bool drawFirst = getAttribBool(XMLNode, "drawFirst", true);
-    bool active = getAttribBool(XMLNode, "active", false);
-    if (!active)
-        return;
 
     // Process rotation (?)
     Quaternion rotation = Quaternion::IDENTITY;
@@ -847,6 +865,16 @@ void DotSceneLoader::processLightAttenuation(pugi::xml_node& XMLNode, Light* pLi
     pLight->setAttenuation(range, constant, linear, quadratic);
 }
 
+void DotSceneLoader::processLightSourceSize(pugi::xml_node& XMLNode, Light* pLight)
+{
+    // Process attributes
+    Real width = getAttribReal(XMLNode, "width");
+    Real height = getAttribReal(XMLNode, "height");
+
+    // Setup the light range
+    pLight->setSourceSize(width, height);
+}
+
 void DotSceneLoader::processUserData(pugi::xml_node& XMLNode, UserObjectBindings& userData)
 {
     // Process node (*)
@@ -917,6 +945,7 @@ void DotSceneLoader::processNodeAnimation(pugi::xml_node& XMLNode, SceneNode* pP
 
     // create a track to animate the camera's node
     NodeAnimationTrack* track = anim->createNodeTrack(0, pParent);
+    pParent->setInitialState();
 
     // Process keyframes (*)
     for (auto pElement : XMLNode.children("keyframe"))
@@ -1108,7 +1137,8 @@ const Ogre::String& DotScenePlugin::getName() const {
     return name;
 }
 
-void DotScenePlugin::initialise() {
+void DotScenePlugin::install()
+{
     mCodec = new DotSceneCodec();
     Codec::registerCodec(mCodec);
 }
