@@ -31,10 +31,8 @@ THE SOFTWARE.
 #endif
 #include "OgreWin32Window.h"
 #include "OgreRoot.h"
-#include "OgreViewport.h"
 #include "OgreLogManager.h"
 #include "OgreRenderSystem.h"
-#include "OgreImageCodec.h"
 #include "OgreStringConverter.h"
 #include "OgreException.h"
 #include "OgreWin32GLSupport.h"
@@ -94,7 +92,6 @@ namespace Ogre {
         mIsFullScreen = fullScreen;
         mClosed = false;        
         mDisplayFrequency = 0;
-        mDepthBufferPoolId = DepthBuffer::POOL_DEFAULT;
         mColourDepth = mIsFullScreen? 32 : GetDeviceCaps(GetDC(0), BITSPIXEL);
         int left = -1; // Defaults to screen center
         int top = -1; // Defaults to screen center
@@ -125,8 +122,8 @@ namespace Ogre {
 
             if ((opt = miscParams->find("depthBuffer")) != end)
             {
-                mDepthBufferPoolId = StringConverter::parseBool(opt->second) ?
-                                                DepthBuffer::POOL_DEFAULT : DepthBuffer::POOL_NO_DEPTH;
+                LogManager::getSingleton().logWarning(
+                    "Win32Window::create(): depthBuffer option is not supported on OpenGL.");
             }
 
             if ((opt = miscParams->find("vsync")) != end)
@@ -751,16 +748,9 @@ namespace Ogre {
         unsigned int width = rc.right - rc.left;
         unsigned int height = rc.bottom - rc.top;
 
-        // Case window resized.
         if (width != mWidth || height != mHeight)
         {
-            mWidth  = rc.right - rc.left;
-            mHeight = rc.bottom - rc.top;
-
-            // Notify viewports of resize
-            ViewportList::iterator it = mViewportList.begin();
-            while( it != mViewportList.end() )
-                (*it++).second->_updateDimensions();            
+            RenderWindow::resize(rc.right - rc.left, rc.bottom - rc.top);
         }
     }
 
