@@ -92,7 +92,6 @@ PlaypenTestPlugin::PlaypenTestPlugin()
     addSample(new PlayPen_MultiSceneManagersSimple()); // upper right only has ambient lighting, ignores light source
     addSample(new PlayPen_MultiViewports());
     addSample(new PlayPen_NegativeScale());
-    addSample(new PlayPen_NonUniqueResourceNames());
     addSample(new PlayPen_NormalMapMirroredUVs()); // texture is missing
     addSample(new PlayPen_Ortho());
     addSample(new PlayPen_PBR());
@@ -109,7 +108,6 @@ PlaypenTestPlugin::PlaypenTestPlugin()
     addSample(new PlayPen_RibbonTrail());
     addSample(new PlayPen_SRGBtexture()); // bug on Win
     addSample(new PlayPen_SceneNodeTracking());
-    addSample(new PlayPen_SerialisedColour());
     addSample(new PlayPen_ShadowLod()); // causes bugs?
     addSample(new PlayPen_SkeletalAnimation());
 
@@ -130,7 +128,7 @@ PlaypenTestPlugin::PlaypenTestPlugin()
 
     addSample(new PlayPen_TransparencyMipMaps());
     addSample(new PlayPen_TwoSidedLighting());
-    addSample(new PlayPen_VertexTexture()); // needs to be implemented in OpenGL
+    if(!only_common) addSample(new PlayPen_VertexTexture()); // needs to be implemented in OpenGLES
     addSample(new PlayPen_ViewportNoShadows());
     addSample(new PlayPen_WindowedViewportMode()); // possibly should only show quarter of screen?
     addSample(new PlayPen_16Textures());
